@@ -432,6 +432,25 @@ func (c *Connection) Vacuum(ctx context.Context, pool, revision string, dryrun b
 	return res, err
 }
 
+func (c *Connection) AddVectors(ctx context.Context, pool, revision string, objectIDs []ksuid.KSUID, message api.CommitMessage) (api.CommitResponse, error) {
+	return c.doVector(ctx, pool, revision, objectIDs, message, http.MethodPost)
+}
+
+func (c *Connection) DeleteVectors(ctx context.Context, pool, revision string, objectIDs []ksuid.KSUID, message api.CommitMessage) (api.CommitResponse, error) {
+	return c.doVector(ctx, pool, revision, objectIDs, message, http.MethodDelete)
+}
+
+func (c *Connection) doVector(ctx context.Context, pool, revision string, objectIDs []ksuid.KSUID, message api.CommitMessage, method string) (api.CommitResponse, error) {
+	path := urlPath("pool", pool, "revision", revision, "vector")
+	req := c.NewRequest(ctx, method, path, api.VectorRequest{ObjectIDs: objectIDs})
+	if err := encodeCommitMessage(req, message); err != nil {
+		return api.CommitResponse{}, err
+	}
+	var res api.CommitResponse
+	err := c.doAndUnmarshal(req, &res)
+	return res, err
+}
+
 func (c *Connection) SubscribeEvents(ctx context.Context) (*EventsClient, error) {
 	req := c.NewRequest(ctx, http.MethodGet, "/events", nil)
 	req.Header.Set("Accept", api.MediaTypeZSON)
