@@ -12,6 +12,7 @@ type Run struct {
 	extent.Span
 	Compare expr.CompareFn
 	Objects []*data.Object
+	Size    int64
 }
 
 func NewRun(cmp expr.CompareFn) Run {
@@ -27,6 +28,7 @@ func (p Run) Overlaps(first, last *zed.Value) bool {
 
 func (p *Run) Add(o *data.Object) {
 	p.Objects = append(p.Objects, o)
+	p.Size += o.Size
 	if p.Span == nil {
 		p.Span = extent.NewGeneric(o.Min, o.Max, p.Compare)
 		return
@@ -35,16 +37,6 @@ func (p *Run) Add(o *data.Object) {
 	p.Span.Extend(&o.Max)
 }
 
-func (p *Run) SizeUncoveredBy(span extent.Span) int64 {
-	var size int64
-	for _, o := range p.Objects {
-		if span == nil || !span.Overlaps(&o.Min, &o.Max) {
-			size += o.Size
-		}
-	}
-	return size
-}
-
 func (p *Run) ObjectIDs() []ksuid.KSUID {
 	var ids []ksuid.KSUID
 	for _, o := range p.Objects {
