@@ -51,6 +51,7 @@ var unpacker = unpack.New(
 	Pool{},
 	astzed.Primitive{},
 	Put{},
+	QuotedString{},
 	astzed.Record{},
 	Agg{},
 	Regexp{},
@@ -66,6 +67,7 @@ var unpacker = unpack.New(
 	SQLOrderBy{},
 	Sort{},
 	String{},
+	OpDecl{},
 	Switch{},
 	Tail{},
 	Term{},
