@@ -4,6 +4,7 @@ import (
 	"context"
 	"errors"
 	"fmt"
+	"strings"
 	"sync"
 
 	"github.com/brimdata/zed"
@@ -38,6 +39,7 @@ import (
 	"github.com/brimdata/zed/zio"
 	"github.com/brimdata/zed/zson"
 	"github.com/segmentio/ksuid"
+	"golang.org/x/exp/slices"
 )
 
 var ErrJoinParents = errors.New("join requires two upstream parallel query paths")
@@ -259,7 +261,8 @@ func (b *Builder) compileLeaf(o dag.Op, parent zbuf.Puller) (zbuf.Puller, error)
 	case *dag.LakeMetaScan:
 		return meta.NewLakeMetaScanner(b.octx.Context, b.octx.Zctx, b.source.Lake(), v.Meta)
 	case *dag.HTTPScan:
-		return b.source.Open(b.octx.Context, b.octx.Zctx, v.URL, v.Format, nil)
+		body := strings.NewReader(v.Body)
+		return b.source.OpenHTTP(b.octx.Context, b.octx.Zctx, v.URL, v.Format, v.Method, v.Headers, body)
 	case *dag.FileScan:
 		return b.source.Open(b.octx.Context, b.octx.Zctx, v.Path, v.Format, b.PushdownOf(v.Filter))
 	case *Reader:
@@ -688,15 +691,9 @@ func isEntry(seq dag.Seq) bool {
 	case *dag.Scope:
 		return isEntry(op.Body)
 	case *dag.Fork:
-		if len(op.Paths) == 0 {
-			return false
-		}
-		for _, seq := range op.Paths {
-			if !isEntry(seq) {
-				return false
-			}
-		}
-		return true
+		return len(op.Paths) > 0 && !slices.ContainsFunc(op.Paths, func(seq dag.Seq) bool {
+			return !isEntry(seq)
+		})
 	}
 	return false
 }
