@@ -4,13 +4,16 @@ import (
 	"errors"
 	"fmt"
 	"regexp"
+	"strings"
 
+	"github.com/brimdata/zed"
 	"github.com/brimdata/zed/compiler/ast"
 	"github.com/brimdata/zed/compiler/ast/dag"
 	"github.com/brimdata/zed/compiler/kernel"
 	"github.com/brimdata/zed/lakeparse"
 	"github.com/brimdata/zed/order"
 	"github.com/brimdata/zed/pkg/field"
+	"github.com/brimdata/zed/pkg/plural"
 	"github.com/brimdata/zed/pkg/reglob"
 	"github.com/brimdata/zed/runtime/expr/function"
 	"github.com/brimdata/zed/zson"
@@ -86,10 +89,22 @@ func (a *analyzer) semSource(source ast.Source) ([]dag.Op, error) {
 		if err != nil {
 			return nil, err
 		}
+		var path string
+		switch p := p.Path.(type) {
+		case *ast.QuotedString:
+			path = p.Text
+		case *ast.String:
+			// This can be either a reference to a constant or a string.
+			if path, err = a.maybeStringConst(p.Text); err != nil {
+				return nil, fmt.Errorf("invalid file path: %w", err)
+			}
+		default:
+			return nil, fmt.Errorf("semantic analyzer: unknown AST file type %T", p)
+		}
 		return []dag.Op{
 			&dag.FileScan{
 				Kind:    "FileScan",
-				Path:    p.Path,
+				Path:    path,
 				Format:  p.Format,
 				SortKey: sortKey,
 			},
@@ -99,12 +114,45 @@ func (a *analyzer) semSource(source ast.Source) ([]dag.Op, error) {
 		if err != nil {
 			return nil, err
 		}
+		var headers map[string][]string
+		if p.Headers != nil {
+			expr, err := a.semExpr(p.Headers)
+			if err != nil {
+				return nil, err
+			}
+			val, err := kernel.EvalAtCompileTime(a.zctx, expr)
+			if err != nil {
+				return nil, fmt.Errorf("headers: %w", err)
+			}
+			headers, err = unmarshalHeaders(val)
+			if err != nil {
+				return nil, err
+			}
+		}
+		var url string
+		switch p := p.URL.(type) {
+		case *ast.QuotedString:
+			url = p.Text
+		case *ast.String:
+			// This can be either a reference to a constant or a string.
+			if url, err = a.maybeStringConst(p.Text); err != nil {
+				return nil, fmt.Errorf("invalid file path: %w", err)
+			}
+		default:
+			return nil, fmt.Errorf("semantic analyzer: unsupported AST get type %T", p)
+		}
+		if !strings.HasPrefix(url, "http://") && !strings.HasPrefix(url, "https://") {
+			return nil, fmt.Errorf("get: invalid URL %s", url)
+		}
 		return []dag.Op{
 			&dag.HTTPScan{
 				Kind:    "HTTPScan",
-				URL:     p.URL,
+				URL:     url,
 				Format:  p.Format,
 				SortKey: sortKey,
+				Method:  p.Method,
+				Headers: headers,
+				Body:    p.Body,
 			},
 		}, nil
 	case *ast.Pool:
@@ -122,7 +170,27 @@ func (a *analyzer) semSource(source ast.Source) ([]dag.Op, error) {
 		return nil, fmt.Errorf("semantic analyzer: unknown AST source type %T", p)
 	}
 }
-
+func unmarshalHeaders(val *zed.Value) (map[string][]string, error) {
+	if !zed.IsRecordType(val.Type) {
+		return nil, errors.New("headers value must be a record")
+	}
+	headers := map[string][]string{}
+	for i, f := range val.Fields() {
+		if inner := zed.InnerType(f.Type); inner == nil || inner.ID() != zed.IDString {
+			return nil, errors.New("headers field value must be an array or set of strings")
+		}
+		fieldVal := val.DerefByColumn(i)
+		if fieldVal == nil {
+			continue
+		}
+		for it := fieldVal.Iter(); !it.Done(); {
+			if b := it.Next(); b != nil {
+				headers[f.Name] = append(headers[f.Name], zed.DecodeString(b))
+			}
+		}
+	}
+	return headers, nil
+}
 func semSortKey(p *ast.SortKey) (order.SortKey, error) {
 	if p == nil || p.Keys == nil {
 		return order.Nil, nil
@@ -154,6 +222,13 @@ func (a *analyzer) semPool(p *ast.Pool) ([]dag.Op, error) {
 	case *ast.Regexp:
 		poolNames, err = a.matchPools(specPool.Pattern, specPool.Pattern, "regexp")
 	case *ast.String:
+		// This can be either a reference to a constant or a string.
+		name, err := a.maybeStringConst(specPool.Text)
+		if err != nil {
+			return nil, fmt.Errorf("invalid pool name: %w", err)
+		}
+		poolNames = []string{name}
+	case *ast.QuotedString:
 		poolNames = []string{specPool.Text}
 	default:
 		return nil, fmt.Errorf("semantic analyzer: unknown AST pool type %T", specPool)
@@ -172,6 +247,22 @@ func (a *analyzer) semPool(p *ast.Pool) ([]dag.Op, error) {
 	return sources, nil
 }
 
+func (a *analyzer) maybeStringConst(name string) (string, error) {
+	e, err := a.scope.LookupExpr(name)
+	if err != nil || e == nil {
+		return name, err
+	}
+	l, ok := e.(*dag.Literal)
+	if !ok {
+		return "", fmt.Errorf("%s: string value required", name)
+	}
+	val := zson.MustParseValue(a.zctx, l.Value)
+	if val.Type.ID() != zed.IDString {
+		return "", fmt.Errorf("%s: string value required", name)
+	}
+	return val.AsString(), nil
+}
+
 func (a *analyzer) semPoolWithName(p *ast.Pool, poolName string) (dag.Op, error) {
 	commit := p.Spec.Commit
 	if poolName == "HEAD" {
@@ -291,8 +382,8 @@ func (a *analyzer) matchPools(pattern, origPattern, patternDesc string) ([]strin
 }
 
 func (a *analyzer) semScope(op *ast.Scope) (*dag.Scope, error) {
-	a.scope.Enter()
-	defer a.scope.Exit()
+	a.scope = NewScope(a.scope)
+	defer a.exitScope()
 	consts, funcs, err := a.semDecls(op.Decls)
 	if err != nil {
 		return nil, err
@@ -447,7 +538,7 @@ func (a *analyzer) semOp(o ast.Op, seq dag.Seq) (dag.Seq, error) {
 			return nil, fmt.Errorf("head: %w", err)
 		}
 		if val.AsInt() < 1 {
-			return nil, fmt.Errorf("head: expression value is not a positive integer: %s", zson.MustFormatValue(val))
+			return nil, fmt.Errorf("head: expression value is not a positive integer: %s", zson.FormatValue(val))
 		}
 		return append(seq, &dag.Head{
 			Kind:  "Head",
@@ -463,7 +554,7 @@ func (a *analyzer) semOp(o ast.Op, seq dag.Seq) (dag.Seq, error) {
 			return nil, fmt.Errorf("tail: %w", err)
 		}
 		if val.AsInt() < 1 {
-			return nil, fmt.Errorf("tail: expression value is not a positive integer: %s", zson.MustFormatValue(val))
+			return nil, fmt.Errorf("tail: expression value is not a positive integer: %s", zson.FormatValue(val))
 		}
 		return append(seq, &dag.Tail{
 			Kind:  "Tail",
@@ -633,8 +724,8 @@ func (a *analyzer) semOp(o ast.Op, seq dag.Seq) (dag.Seq, error) {
 		if len(o.Locals) != 0 && o.Body == nil {
 			return nil, errors.New("over operator: cannot have a with clause without a lateral query")
 		}
-		a.scope.Enter()
-		defer a.scope.Exit()
+		a.enterScope()
+		defer a.exitScope()
 		locals, err := a.semVars(o.Locals)
 		if err != nil {
 			return nil, err
@@ -736,7 +827,7 @@ func (a *analyzer) singletonAgg(agg ast.Assignment, seq dag.Seq) dag.Seq {
 
 func (a *analyzer) semDecls(decls []ast.Decl) ([]dag.Def, []*dag.Func, error) {
 	var consts []dag.Def
-	var fds []*ast.FuncDecl
+	var fnDecls []*ast.FuncDecl
 	for _, d := range decls {
 		switch d := d.(type) {
 		case *ast.ConstDecl:
@@ -746,12 +837,16 @@ func (a *analyzer) semDecls(decls []ast.Decl) ([]dag.Def, []*dag.Func, error) {
 			}
 			consts = append(consts, c)
 		case *ast.FuncDecl:
-			fds = append(fds, d)
+			fnDecls = append(fnDecls, d)
+		case *ast.OpDecl:
+			if err := a.semOpDecl(d); err != nil {
+				return nil, nil, err
+			}
 		default:
 			return nil, nil, fmt.Errorf("invalid declaration type %T", d)
 		}
 	}
-	funcs, err := a.semFuncDecls(fds)
+	funcs, err := a.semFuncDecls(fnDecls)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -780,7 +875,7 @@ func (a *analyzer) semFuncDecls(decls []*ast.FuncDecl) ([]*dag.Func, error) {
 			Name:   d.Name,
 			Params: slices.Clone(d.Params),
 		}
-		if err := a.scope.DefineFunc(f); err != nil {
+		if err := a.scope.DefineAs(f.Name, f); err != nil {
 			return nil, err
 		}
 		funcs = append(funcs, f)
@@ -795,8 +890,8 @@ func (a *analyzer) semFuncDecls(decls []*ast.FuncDecl) ([]*dag.Func, error) {
 }
 
 func (a *analyzer) semFuncBody(params []string, body ast.Expr) (dag.Expr, error) {
-	a.scope.Enter()
-	defer a.scope.Exit()
+	a.enterScope()
+	defer a.exitScope()
 	for _, p := range params {
 		if err := a.scope.DefineVar(p); err != nil {
 			return nil, err
@@ -805,6 +900,17 @@ func (a *analyzer) semFuncBody(params []string, body ast.Expr) (dag.Expr, error)
 	return a.semExpr(body)
 }
 
+func (a *analyzer) semOpDecl(d *ast.OpDecl) error {
+	m := make(map[string]bool)
+	for _, p := range d.Params {
+		if m[p] {
+			return fmt.Errorf("%s: duplicate parameter %q", d.Name, p)
+		}
+		m[p] = true
+	}
+	return a.scope.DefineAs(d.Name, &opDecl{ast: d, scope: a.scope})
+}
+
 func (a *analyzer) semVars(defs []ast.Def) ([]dag.Def, error) {
 	var locals []dag.Def
 	for _, def := range defs {
@@ -908,6 +1014,11 @@ func isBool(e dag.Expr) bool {
 }
 
 func (a *analyzer) semCallOp(call *ast.Call, seq dag.Seq) (dag.Seq, error) {
+	if body, err := a.maybeConvertUserOp(call, seq); err != nil {
+		return nil, err
+	} else if body != nil {
+		return append(seq, body...), nil
+	}
 	if agg, err := a.maybeConvertAgg(call); err == nil && agg != nil {
 		summarize := &dag.Summarize{
 			Kind: "Summarize",
@@ -934,3 +1045,61 @@ func (a *analyzer) semCallOp(call *ast.Call, seq dag.Seq) (dag.Seq, error) {
 	}
 	return append(seq, dag.NewFilter(c)), nil
 }
+
+// maybeConvertUserOp returns nil, nil if the call is determined to not be a
+// UserOp, otherwise it returns the compiled op or the encountered error.
+func (a *analyzer) maybeConvertUserOp(call *ast.Call, seq dag.Seq) (dag.Seq, error) {
+	decl, err := a.scope.lookupOp(call.Name)
+	if decl == nil || err != nil {
+		return nil, nil
+	}
+	if call.Where != nil {
+		return nil, fmt.Errorf("%s(): user defined operators cannot have a where clause", call.Name)
+	}
+	params, args := decl.ast.Params, call.Args
+	if len(params) != len(args) {
+		return nil, fmt.Errorf("%s(): %d arg%s provided when operator expects %d arg%s", call.Name, len(params), plural.Slice(params, "s"), len(args), plural.Slice(args, "s"))
+	}
+	exprs := make([]dag.Expr, len(decl.ast.Params))
+	for i, arg := range args {
+		e, err := a.semExpr(arg)
+		if err != nil {
+			return nil, err
+		}
+		// Transform non-path arguments into literals.
+		if _, ok := e.(*dag.This); !ok {
+			val, err := kernel.EvalAtCompileTime(a.zctx, e)
+			if err != nil {
+				return nil, err
+			}
+			if val.IsError() {
+				if val.IsMissing() {
+					return nil, fmt.Errorf("%q: non-path arguments cannot have variable dependency", decl.ast.Params[i])
+				} else {
+					return nil, fmt.Errorf("%q: %q", decl.ast.Params[i], string(val.Bytes()))
+				}
+			}
+			e = &dag.Literal{
+				Kind:  "Literal",
+				Value: zson.FormatValue(val),
+			}
+		}
+		exprs[i] = e
+	}
+	if slices.Contains(a.opStack, decl.ast) {
+		return nil, opCycleError(append(a.opStack, decl.ast))
+	}
+	a.opStack = append(a.opStack, decl.ast)
+	oldscope := a.scope
+	a.scope = NewScope(decl.scope)
+	defer func() {
+		a.opStack = a.opStack[:len(a.opStack)-1]
+		a.scope = oldscope
+	}()
+	for i, p := range params {
+		if err := a.scope.DefineAs(p, exprs[i]); err != nil {
+			return nil, err
+		}
+	}
+	return a.semSeq(decl.ast.Body)
+}
