@@ -209,7 +209,11 @@ func (a *analyzer) convertSQLTableRef(e ast.Expr) (dag.Op, error) {
 	// then convert it to a type name as it is otherwise expected that
 	// the type name will be defined by the data stream.
 	if id, ok := dag.TopLevelField(converted); ok {
-		if a.scope.Lookup(id) == nil {
+		e, err := a.scope.LookupExpr(id)
+		if err != nil {
+			return nil, err
+		}
+		if e == nil {
 			converted = dynamicTypeName(id)
 		}
 	}
