@@ -229,16 +229,24 @@ func (l *local) UpdateIndex(ctx context.Context, ruleRefs []string, poolID ksuid
 	return branch.UpdateIndex(ctx, l.compiler, rules)
 }
 
-func (l *local) AddVectors(ctx context.Context, poolID ksuid.KSUID, branchName string, ids []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error) {
-	_, branch, err := l.lookupBranch(ctx, poolID, branchName)
+func (l *local) AddVectors(ctx context.Context, pool, revision string, ids []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error) {
+	poolID, err := l.PoolID(ctx, pool)
+	if err != nil {
+		return ksuid.Nil, err
+	}
+	_, branch, err := l.lookupBranch(ctx, poolID, revision)
 	if err != nil {
 		return ksuid.Nil, err
 	}
 	return branch.AddVectors(ctx, ids, message.Author, message.Body)
 }
 
-func (l *local) DeleteVectors(ctx context.Context, poolID ksuid.KSUID, branchName string, ids []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error) {
-	_, branch, err := l.lookupBranch(ctx, poolID, branchName)
+func (l *local) DeleteVectors(ctx context.Context, pool, revision string, ids []ksuid.KSUID, message api.CommitMessage) (ksuid.KSUID, error) {
+	poolID, err := l.PoolID(ctx, pool)
+	if err != nil {
+		return ksuid.Nil, err
+	}
+	_, branch, err := l.lookupBranch(ctx, poolID, revision)
 	if err != nil {
 		return ksuid.Nil, err
 	}
