@@ -28,14 +28,14 @@ func LookupSeekRange(ctx context.Context, engine storage.Engine, path *storage.U
 	unmarshaler := zson.NewZNGUnmarshaler()
 	reader := zngio.NewReader(zed.NewContext(), r)
 	defer reader.Close()
-	ectx := expr.NewContext()
+	var ectx expr.ResetContext
 	for {
 		val, err := reader.Read()
 		if val == nil || err != nil {
 			return ranges, err
 		}
-		result := pruner.Eval(ectx, val)
-		if result.Type == zed.TypeBool && zed.DecodeBool(result.Bytes) {
+		result := pruner.Eval(ectx.Reset(), val)
+		if result.Type == zed.TypeBool && result.Bool() {
 			rg = nil
 			continue
 		}
