@@ -77,7 +77,7 @@ func (d Direction) MarshalZNG(m *zson.MarshalZNGContext) (zed.Type, error) {
 }
 
 func (d *Direction) UnmarshalZNG(u *zson.UnmarshalZNGContext, val *zed.Value) error {
-	dir, err := ParseDirection(string(val.Bytes))
+	dir, err := ParseDirection(string(val.Bytes()))
 	if err != nil {
 		return err
 	}
