@@ -6,6 +6,7 @@ import (
 	"github.com/brimdata/zed"
 	"github.com/brimdata/zed/pkg/field"
 	"github.com/brimdata/zed/zcode"
+	"golang.org/x/exp/slices"
 )
 
 type This struct{}
@@ -15,9 +16,10 @@ func (*This) Eval(_ Context, this *zed.Value) *zed.Value {
 }
 
 type DotExpr struct {
-	zctx   *zed.Context
-	record Evaluator
-	field  string
+	zctx         *zed.Context
+	record       Evaluator
+	field        string
+	fieldIndices []int
 }
 
 func NewDotExpr(zctx *zed.Context, record Evaluator, field string) *DotExpr {
@@ -38,28 +40,39 @@ func NewDottedExpr(zctx *zed.Context, f field.Path) Evaluator {
 
 func (d *DotExpr) Eval(ectx Context, this *zed.Value) *zed.Value {
 	val := d.record.Eval(ectx, this).Under()
-	if _, ok := val.Type.(*zed.TypeOfType); ok {
-		return d.evalTypeOfType(ectx, val.Bytes)
+	// Cases are ordered by decreasing expected frequency.
+	switch typ := val.Type.(type) {
+	case *zed.TypeRecord:
+		i, ok := d.fieldIndex(typ)
+		if !ok {
+			return d.zctx.Missing()
+		}
+		return ectx.NewValue(typ.Fields[i].Type, getNthFromContainer(val.Bytes(), i))
+	case *zed.TypeMap:
+		return indexMap(d.zctx, ectx, typ, val.Bytes(), zed.NewString(d.field))
+	case *zed.TypeOfType:
+		return d.evalTypeOfType(ectx, val.Bytes())
 	}
-	if typ, ok := val.Type.(*zed.TypeMap); ok {
-		return indexMap(d.zctx, ectx, typ, val.Bytes, zed.NewString(d.field))
+	return d.zctx.Missing()
+}
+
+func (d *DotExpr) fieldIndex(typ *zed.TypeRecord) (int, bool) {
+	id := typ.ID()
+	if id >= len(d.fieldIndices) {
+		d.fieldIndices = slices.Grow(d.fieldIndices[:0], id+1)[:id+1]
 	}
-	recType, ok := val.Type.(*zed.TypeRecord)
-	if !ok {
-		return d.zctx.Missing()
+	if i := d.fieldIndices[id]; i > 0 {
+		return i - 1, true
+	} else if i < 0 {
+		return 0, false
 	}
-	idx, ok := recType.ColumnOfField(d.field)
-	if !ok {
-		return d.zctx.Missing()
+	i, ok := typ.IndexOfField(d.field)
+	if ok {
+		d.fieldIndices[id] = i + 1
+	} else {
+		d.fieldIndices[id] = -1
 	}
-	typ := recType.Fields[idx].Type
-	if val.IsNull() {
-		// The record is null.  Return null value of the field type.
-		return ectx.NewValue(typ, nil)
-	}
-	//XXX see PR #1071 to improve this (though we need this for Index anyway)
-	field := getNthFromContainer(val.Bytes, idx)
-	return ectx.NewValue(recType.Fields[idx].Type, field)
+	return i, ok
 }
 
 func (d *DotExpr) evalTypeOfType(ectx Context, b zcode.Bytes) *zed.Value {
