@@ -38,7 +38,7 @@ func (n *Not) Eval(ectx Context, this *zed.Value) *zed.Value {
 	if !ok {
 		return val
 	}
-	if zed.DecodeBool(val.Bytes) {
+	if val.Bool() {
 		return zed.False
 	}
 	return zed.True
@@ -75,7 +75,7 @@ func EvalBool(zctx *zed.Context, ectx Context, this *zed.Value, e Evaluator) (*z
 	if val.IsError() {
 		return val, false
 	}
-	return ectx.CopyValue(zctx.NewErrorf("not type bool: %s", zson.MustFormatValue(val))), false
+	return ectx.CopyValue(*zctx.WrapError("not type bool", val)), false
 }
 
 func (a *And) Eval(ectx Context, this *zed.Value) *zed.Value {
@@ -83,14 +83,14 @@ func (a *And) Eval(ectx Context, this *zed.Value) *zed.Value {
 	if !ok {
 		return lhs
 	}
-	if !zed.DecodeBool(lhs.Bytes) {
+	if !lhs.Bool() {
 		return zed.False
 	}
 	rhs, ok := EvalBool(a.zctx, ectx, this, a.rhs)
 	if !ok {
 		return rhs
 	}
-	if !zed.DecodeBool(rhs.Bytes) {
+	if !rhs.Bool() {
 		return zed.False
 	}
 	return zed.True
@@ -98,7 +98,7 @@ func (a *And) Eval(ectx Context, this *zed.Value) *zed.Value {
 
 func (o *Or) Eval(ectx Context, this *zed.Value) *zed.Value {
 	lhs, ok := EvalBool(o.zctx, ectx, this, o.lhs)
-	if ok && zed.DecodeBool(lhs.Bytes) {
+	if ok && lhs.Bool() {
 		return zed.True
 	}
 	if lhs.IsError() && !lhs.IsMissing() {
@@ -106,7 +106,7 @@ func (o *Or) Eval(ectx Context, this *zed.Value) *zed.Value {
 	}
 	rhs, ok := EvalBool(o.zctx, ectx, this, o.rhs)
 	if ok {
-		if zed.DecodeBool(rhs.Bytes) {
+		if rhs.Bool() {
 			return zed.True
 		}
 		return zed.False
@@ -138,8 +138,10 @@ func (i *In) Eval(ectx Context, this *zed.Value) *zed.Value {
 	if container.IsError() {
 		return container
 	}
+	tmpVal := ectx.NewValue(nil, nil)
 	err := container.Walk(func(typ zed.Type, body zcode.Bytes) error {
-		if _, err := i.vals.Coerce(elem, zed.NewValue(typ, body)); err != nil {
+		*tmpVal = *zed.NewValue(typ, body)
+		if _, err := i.vals.Coerce(elem, tmpVal); err != nil {
 			if err != coerce.IncompatibleTypes {
 				return err
 			}
@@ -163,8 +165,8 @@ type Equal struct {
 	equality bool
 }
 
-func NewCompareEquality(lhs, rhs Evaluator, operator string) (*Equal, error) {
-	e := &Equal{numeric: newNumeric(lhs, rhs)} //XXX
+func NewCompareEquality(zctx *zed.Context, lhs, rhs Evaluator, operator string) (*Equal, error) {
+	e := &Equal{numeric: newNumeric(zctx, lhs, rhs)} //XXX
 	switch operator {
 	case "==":
 		e.equality = true
@@ -181,19 +183,15 @@ func (e *Equal) Eval(ectx Context, this *zed.Value) *zed.Value {
 		return zerr
 	}
 	if err != nil {
-		switch {
-		case err == coerce.Overflow:
-			// If there was overflow converting one to the other,
-			// we know they can't be equal.
+		if errors.Is(err, coerce.IncompatibleTypes) || errors.Is(err, coerce.Overflow) {
+			// If the types are incompatible or there was overflow,
+			// then, then we know the values can't be equal.
 			if e.equality {
 				return zed.False
 			}
 			return zed.True
-		case err == coerce.IncompatibleTypes:
-			return zed.False
-		default:
-			return e.zctx.NewError(err)
 		}
+		return e.zctx.NewError(err)
 	}
 	result := e.vals.Equal()
 	if !e.equality {
@@ -216,7 +214,7 @@ func NewRegexpMatch(re *regexp.Regexp, e Evaluator) *RegexpMatch {
 
 func (r *RegexpMatch) Eval(ectx Context, this *zed.Value) *zed.Value {
 	val := r.expr.Eval(ectx, this)
-	if val.Type.ID() == zed.IDString && r.re.Match(val.Bytes) {
+	if val.Type.ID() == zed.IDString && r.re.Match(val.Bytes()) {
 		return zed.True
 	}
 	return zed.False
@@ -229,17 +227,18 @@ type numeric struct {
 	vals coerce.Pair
 }
 
-func newNumeric(lhs, rhs Evaluator) numeric {
+func newNumeric(zctx *zed.Context, lhs, rhs Evaluator) numeric {
 	return numeric{
-		lhs: lhs,
-		rhs: rhs,
+		zctx: zctx,
+		lhs:  lhs,
+		rhs:  rhs,
 	}
 }
 
-func enumify(val *zed.Value) *zed.Value {
+func enumify(ectx Context, val *zed.Value) *zed.Value {
 	// automatically convert an enum to its index value when coercing
 	if _, ok := val.Type.(*zed.TypeEnum); ok {
-		return zed.NewValue(zed.TypeUint64, val.Bytes)
+		return ectx.NewValue(zed.TypeUint64, val.Bytes())
 	}
 	return val
 }
@@ -249,12 +248,12 @@ func (n *numeric) eval(ectx Context, this *zed.Value) (int, *zed.Value, error) {
 	if lhs.IsError() {
 		return 0, lhs, nil
 	}
-	lhs = enumify(lhs)
+	lhs = enumify(ectx, lhs)
 	rhs := n.rhs.Eval(ectx, this)
 	if rhs.IsError() {
 		return 0, rhs, nil
 	}
-	rhs = enumify(rhs)
+	rhs = enumify(ectx, rhs)
 	id, err := n.vals.Coerce(lhs, rhs)
 	return id, nil, err
 }
@@ -278,7 +277,7 @@ type Compare struct {
 }
 
 func NewCompareRelative(zctx *zed.Context, lhs, rhs Evaluator, operator string) (*Compare, error) {
-	c := &Compare{zctx: zctx, numeric: newNumeric(lhs, rhs)}
+	c := &Compare{zctx: zctx, numeric: newNumeric(zctx, lhs, rhs)}
 	switch operator {
 	case "<":
 		c.convert = func(v int) bool { return v < 0 }
@@ -360,7 +359,7 @@ func (c *Compare) Eval(ectx Context, this *zed.Value) *zed.Value {
 				result = 1
 			}
 		default:
-			return ectx.CopyValue(c.zctx.NewErrorf("bad comparison type ID: %d", id))
+			return ectx.CopyValue(*c.zctx.NewErrorf("bad comparison type ID: %d", id))
 		}
 	}
 	if c.convert(result) {
@@ -399,7 +398,7 @@ var DivideByZero = errors.New("divide by zero")
 // NewArithmetic compiles an expression of the form "expr1 op expr2"
 // for the arithmetic operators +, -, *, /
 func NewArithmetic(zctx *zed.Context, lhs, rhs Evaluator, op string) (Evaluator, error) {
-	n := newNumeric(lhs, rhs)
+	n := newNumeric(zctx, lhs, rhs)
 	switch op {
 	case "+":
 		return &Add{zctx: zctx, operands: n}, nil
@@ -430,19 +429,19 @@ func (a *Add) Eval(ectx Context, this *zed.Value) *zed.Value {
 	switch {
 	case zed.IsFloat(id):
 		v1, v2 := a.operands.floats()
-		return ectx.NewValue(typ, zed.EncodeFloat64(v1+v2))
+		return ectx.CopyValue(*zed.NewFloat(typ, v1+v2))
 	case zed.IsSigned(id):
 		v1, v2 := a.operands.ints()
-		return ectx.NewValue(typ, zed.EncodeInt(v1+v2))
+		return ectx.CopyValue(*zed.NewInt(typ, v1+v2))
 	case zed.IsNumber(id):
 		v1, v2 := a.operands.uints()
-		return ectx.NewValue(typ, zed.EncodeUint(v1+v2))
+		return ectx.CopyValue(*zed.NewUint(typ, v1+v2))
 	case id == zed.IDString:
 		v1, v2 := zed.DecodeString(a.operands.vals.A), zed.DecodeString(a.operands.vals.B)
 		// XXX GC
 		return ectx.NewValue(typ, zed.EncodeString(v1+v2))
 	}
-	return ectx.CopyValue(a.zctx.NewErrorf("type %s incompatible with '+' operator", zson.FormatType(typ)))
+	return ectx.CopyValue(*a.zctx.NewErrorf("type %s incompatible with '+' operator", zson.FormatType(typ)))
 }
 
 func (s *Subtract) Eval(ectx Context, this *zed.Value) *zed.Value {
@@ -460,19 +459,19 @@ func (s *Subtract) Eval(ectx Context, this *zed.Value) *zed.Value {
 	switch {
 	case zed.IsFloat(id):
 		v1, v2 := s.operands.floats()
-		return ectx.NewValue(typ, zed.EncodeFloat64(v1-v2))
+		return ectx.CopyValue(*zed.NewFloat(typ, v1-v2))
 	case zed.IsSigned(id):
 		v1, v2 := s.operands.ints()
 		if id == zed.IDTime {
 			// Return the difference of two times as a duration.
 			typ = zed.TypeDuration
 		}
-		return ectx.NewValue(typ, zed.EncodeInt(v1-v2))
+		return ectx.CopyValue(*zed.NewInt(typ, v1-v2))
 	case zed.IsNumber(id):
 		v1, v2 := s.operands.uints()
-		return ectx.NewValue(typ, zed.EncodeUint(v1-v2))
+		return ectx.CopyValue(*zed.NewUint(typ, v1-v2))
 	}
-	return ectx.CopyValue(s.zctx.NewErrorf("type %s incompatible with '-' operator", zson.FormatType(typ)))
+	return ectx.CopyValue(*s.zctx.NewErrorf("type %s incompatible with '-' operator", zson.FormatType(typ)))
 }
 
 func (m *Multiply) Eval(ectx Context, this *zed.Value) *zed.Value {
@@ -490,15 +489,15 @@ func (m *Multiply) Eval(ectx Context, this *zed.Value) *zed.Value {
 	switch {
 	case zed.IsFloat(id):
 		v1, v2 := m.operands.floats()
-		return ectx.NewValue(typ, zed.EncodeFloat64(v1*v2))
+		return ectx.CopyValue(*zed.NewFloat(typ, v1*v2))
 	case zed.IsSigned(id):
 		v1, v2 := m.operands.ints()
-		return ectx.NewValue(typ, zed.EncodeInt(v1*v2))
+		return ectx.CopyValue(*zed.NewInt(typ, v1*v2))
 	case zed.IsNumber(id):
 		v1, v2 := m.operands.uints()
-		return ectx.NewValue(typ, zed.EncodeUint(v1*v2))
+		return ectx.CopyValue(*zed.NewUint(typ, v1*v2))
 	}
-	return ectx.CopyValue(m.zctx.NewErrorf("type %s incompatible with '*' operator", zson.FormatType(typ)))
+	return ectx.CopyValue(*m.zctx.NewErrorf("type %s incompatible with '*' operator", zson.FormatType(typ)))
 }
 
 func (d *Divide) Eval(ectx Context, this *zed.Value) *zed.Value {
@@ -519,21 +518,21 @@ func (d *Divide) Eval(ectx Context, this *zed.Value) *zed.Value {
 		if v2 == 0 {
 			return d.zctx.NewError(DivideByZero)
 		}
-		return ectx.NewValue(typ, zed.EncodeFloat64(v1/v2))
+		return ectx.CopyValue(*zed.NewFloat(typ, v1/v2))
 	case zed.IsSigned(id):
 		v1, v2 := d.operands.ints()
 		if v2 == 0 {
 			return d.zctx.NewError(DivideByZero)
 		}
-		return ectx.NewValue(typ, zed.EncodeInt(v1/v2))
+		return ectx.CopyValue(*zed.NewInt(typ, v1/v2))
 	case zed.IsNumber(id):
 		v1, v2 := d.operands.uints()
 		if v2 == 0 {
 			return d.zctx.NewError(DivideByZero)
 		}
-		return ectx.NewValue(typ, zed.EncodeUint(v1/v2))
+		return ectx.CopyValue(*zed.NewUint(typ, v1/v2))
 	}
-	return ectx.CopyValue(d.zctx.NewErrorf("type %s incompatible with '/' operator", zson.FormatType(typ)))
+	return ectx.CopyValue(*d.zctx.NewErrorf("type %s incompatible with '/' operator", zson.FormatType(typ)))
 }
 
 func (m *Modulo) Eval(ectx Context, this *zed.Value) *zed.Value {
@@ -549,20 +548,20 @@ func (m *Modulo) Eval(ectx Context, this *zed.Value) *zed.Value {
 		return m.zctx.NewError(err)
 	}
 	if zed.IsFloat(id) || !zed.IsNumber(id) {
-		return ectx.CopyValue(m.zctx.NewErrorf("type %s incompatible with '%%' operator", zson.FormatType(typ)))
+		return ectx.CopyValue(*m.zctx.NewErrorf("type %s incompatible with '%%' operator", zson.FormatType(typ)))
 	}
 	if zed.IsSigned(id) {
 		x, y := m.operands.ints()
 		if y == 0 {
 			return m.zctx.NewError(DivideByZero)
 		}
-		return ectx.NewValue(typ, zed.EncodeInt(x%y))
+		return ectx.CopyValue(*zed.NewInt(typ, x%y))
 	}
 	x, y := m.operands.uints()
 	if y == 0 {
 		return m.zctx.NewError(DivideByZero)
 	}
-	return ectx.NewValue(typ, zed.EncodeUint(x%y))
+	return ectx.CopyValue(*zed.NewUint(typ, x%y))
 }
 
 type UnaryMinus struct {
@@ -580,94 +579,60 @@ func NewUnaryMinus(zctx *zed.Context, e Evaluator) *UnaryMinus {
 func (u *UnaryMinus) Eval(ectx Context, this *zed.Value) *zed.Value {
 	val := u.expr.Eval(ectx, this)
 	typ := val.Type
+	if val.IsNull() && zed.IsNumber(typ.ID()) {
+		return val
+	}
 	switch typ.ID() {
-	case zed.IDFloat16:
-		if val.IsNull() {
-			return val
-		}
-		return ectx.NewValue(typ, zed.EncodeFloat16(-zed.DecodeFloat16(val.Bytes)))
-	case zed.IDFloat32:
-		if val.IsNull() {
-			return val
-		}
-		return ectx.NewValue(typ, zed.EncodeFloat32(-zed.DecodeFloat32(val.Bytes)))
-	case zed.IDFloat64:
-		if val.IsNull() {
-			return val
-		}
-		return ectx.NewValue(typ, zed.EncodeFloat64(-zed.DecodeFloat64(val.Bytes)))
+	case zed.IDFloat16, zed.IDFloat32, zed.IDFloat64:
+		return ectx.CopyValue(*zed.NewFloat(typ, -val.Float()))
 	case zed.IDInt8:
-		if val.IsNull() {
-			return val
-		}
-		v := zed.DecodeInt(val.Bytes)
+		v := val.Int()
 		if v == math.MinInt8 {
-			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' underflow: int8(%d)", v))
+			return ectx.CopyValue(*u.zctx.WrapError("unary '-' underflow", val))
 		}
-		return ectx.NewValue(typ, zed.EncodeInt(-v))
+		return ectx.CopyValue(*zed.NewInt8(int8(-v)))
 	case zed.IDInt16:
-		if val.IsNull() {
-			return val
-		}
-		v := zed.DecodeInt(val.Bytes)
+		v := val.Int()
 		if v == math.MinInt16 {
-			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' underflow: int16(%d)", v))
+			return ectx.CopyValue(*u.zctx.WrapError("unary '-' underflow", val))
 		}
-		return ectx.NewValue(typ, zed.EncodeInt(-v))
+		return ectx.CopyValue(*zed.NewInt16(int16(-v)))
 	case zed.IDInt32:
-		if val.IsNull() {
-			return val
-		}
-		v := zed.DecodeInt(val.Bytes)
+		v := val.Int()
 		if v == math.MinInt32 {
-			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' underflow: int32(%d)", v))
+			return ectx.CopyValue(*u.zctx.WrapError("unary '-' underflow", val))
 		}
-		return ectx.NewValue(typ, zed.EncodeInt(-v))
+		return ectx.CopyValue(*zed.NewInt32(int32(-v)))
 	case zed.IDInt64:
-		if val.IsNull() {
-			return val
-		}
-		v := zed.DecodeInt(val.Bytes)
+		v := val.Int()
 		if v == math.MinInt64 {
-			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' underflow: int64(%d)", v))
+			return ectx.CopyValue(*u.zctx.WrapError("unary '-' underflow", val))
 		}
-		return ectx.NewValue(typ, zed.EncodeInt(-v))
+		return ectx.CopyValue(*zed.NewInt64(-v))
 	case zed.IDUint8:
-		if val.IsNull() {
-			return val
-		}
-		v := zed.DecodeUint(val.Bytes)
+		v := val.Uint()
 		if v > math.MaxInt8 {
-			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' overflow: uint8(%d)", v))
+			return ectx.CopyValue(*u.zctx.WrapError("unary '-' overflow", val))
 		}
-		return ectx.NewValue(zed.TypeInt8, zed.EncodeInt(-int64(v)))
+		return ectx.CopyValue(*zed.NewInt8(int8(-v)))
 	case zed.IDUint16:
-		if val.IsNull() {
-			return val
-		}
-		v := zed.DecodeUint(val.Bytes)
+		v := val.Uint()
 		if v > math.MaxInt16 {
-			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' overflow: uint16(%d)", v))
+			return ectx.CopyValue(*u.zctx.WrapError("unary '-' overflow", val))
 		}
-		return ectx.NewValue(zed.TypeInt16, zed.EncodeInt(-int64(v)))
+		return ectx.CopyValue(*zed.NewInt16(int16(-v)))
 	case zed.IDUint32:
-		if val.IsNull() {
-			return val
-		}
-		v := zed.DecodeUint(val.Bytes)
+		v := val.Uint()
 		if v > math.MaxInt32 {
-			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' overflow: uint32(%d)", v))
+			return ectx.CopyValue(*u.zctx.WrapError("unary '-' overflow", val))
 		}
-		return ectx.NewValue(zed.TypeInt32, zed.EncodeInt(-int64(v)))
+		return ectx.CopyValue(*zed.NewInt32(int32(-v)))
 	case zed.IDUint64:
-		if val.IsNull() {
-			return val
-		}
-		v := zed.DecodeUint(val.Bytes)
+		v := val.Uint()
 		if v > math.MaxInt64 {
-			return ectx.CopyValue(u.zctx.NewErrorf("unary '-' overflow: uint64(%d)", v))
+			return ectx.CopyValue(*u.zctx.WrapError("unary '-' overflow", val))
 		}
-		return ectx.NewValue(zed.TypeInt64, zed.EncodeInt(-int64(v)))
+		return ectx.CopyValue(*zed.NewInt64(int64(-v)))
 	}
 	return u.zctx.WrapError("type incompatible with unary '-' operator", val)
 }
@@ -696,7 +661,7 @@ func lookupKey(mapBytes, target zcode.Bytes) (zcode.Bytes, bool) {
 	for it := mapBytes.Iter(); !it.Done(); {
 		key := it.Next()
 		val := it.Next()
-		if bytes.Compare(key, target) == 0 {
+		if bytes.Equal(key, target) {
 			return val, true
 		}
 	}
@@ -720,11 +685,11 @@ func (i *Index) Eval(ectx Context, this *zed.Value) *zed.Value {
 	index := i.index.Eval(ectx, this)
 	switch typ := zed.TypeUnder(container.Type).(type) {
 	case *zed.TypeArray, *zed.TypeSet:
-		return indexVector(i.zctx, ectx, zed.InnerType(typ), container.Bytes, index)
+		return indexVector(i.zctx, ectx, zed.InnerType(typ), container.Bytes(), index)
 	case *zed.TypeRecord:
-		return indexRecord(i.zctx, ectx, typ, container.Bytes, index)
+		return indexRecord(i.zctx, ectx, typ, container.Bytes(), index)
 	case *zed.TypeMap:
-		return indexMap(i.zctx, ectx, typ, container.Bytes, index)
+		return indexMap(i.zctx, ectx, typ, container.Bytes(), index)
 	default:
 		return i.zctx.Missing()
 	}
@@ -733,13 +698,13 @@ func (i *Index) Eval(ectx Context, this *zed.Value) *zed.Value {
 func indexVector(zctx *zed.Context, ectx Context, inner zed.Type, vector zcode.Bytes, index *zed.Value) *zed.Value {
 	id := index.Type.ID()
 	if !zed.IsInteger(id) {
-		return ectx.CopyValue(zctx.NewErrorf("array index is not an integer"))
+		return ectx.CopyValue(*zctx.WrapError("array index is not an integer", index))
 	}
 	var idx int
 	if zed.IsSigned(id) {
-		idx = int(zed.DecodeInt(index.Bytes))
+		idx = int(index.Int())
 	} else {
-		idx = int(zed.DecodeUint(index.Bytes))
+		idx = int(index.Uint())
 	}
 	zv := getNthFromContainer(vector, idx)
 	if zv == nil {
@@ -751,14 +716,14 @@ func indexVector(zctx *zed.Context, ectx Context, inner zed.Type, vector zcode.B
 func indexRecord(zctx *zed.Context, ectx Context, typ *zed.TypeRecord, record zcode.Bytes, index *zed.Value) *zed.Value {
 	id := index.Type.ID()
 	if id != zed.IDString {
-		return ectx.CopyValue(zctx.NewErrorf("record index is not a string"))
+		return ectx.CopyValue(*zctx.WrapError("record index is not a string", index))
 	}
-	field := zed.DecodeString(index.Bytes)
-	val := zed.NewValue(typ, record).Deref(field)
+	field := zed.DecodeString(index.Bytes())
+	val := ectx.NewValue(typ, record).Deref(field)
 	if val == nil {
 		return zctx.Missing()
 	}
-	return ectx.CopyValue(val)
+	return ectx.CopyValue(*val)
 }
 
 func indexMap(zctx *zed.Context, ectx Context, typ *zed.TypeMap, mapBytes zcode.Bytes, key *zed.Value) *zed.Value {
@@ -769,7 +734,7 @@ func indexMap(zctx *zed.Context, ectx Context, typ *zed.TypeMap, mapBytes zcode.
 		if union, ok := zed.TypeUnder(typ.KeyType).(*zed.TypeUnion); ok {
 			if tag := union.TagOf(key.Type); tag >= 0 {
 				var b zcode.Builder
-				zed.BuildUnion(&b, union.TagOf(key.Type), key.Bytes)
+				zed.BuildUnion(&b, union.TagOf(key.Type), key.Bytes())
 				if valBytes, ok := lookupKey(mapBytes, b.Bytes().Body()); ok {
 					return deunion(ectx, typ.ValType, valBytes)
 				}
@@ -777,7 +742,7 @@ func indexMap(zctx *zed.Context, ectx Context, typ *zed.TypeMap, mapBytes zcode.
 		}
 		return zctx.Missing()
 	}
-	if valBytes, ok := lookupKey(mapBytes, key.Bytes); ok {
+	if valBytes, ok := lookupKey(mapBytes, key.Bytes()); ok {
 		return deunion(ectx, typ.ValType, valBytes)
 	}
 	return zctx.Missing()
@@ -809,10 +774,10 @@ func NewConditional(zctx *zed.Context, predicate, thenExpr, elseExpr Evaluator)
 func (c *Conditional) Eval(ectx Context, this *zed.Value) *zed.Value {
 	val := c.predicate.Eval(ectx, this)
 	if val.Type.ID() != zed.IDBool {
-		val := *c.zctx.NewErrorf("?-operator: bool predicate required")
+		val := *c.zctx.WrapError("?-operator: bool predicate required", val)
 		return &val
 	}
-	if zed.DecodeBool(val.Bytes) {
+	if val.Bool() {
 		return c.thenExpr.Eval(ectx, this)
 	}
 	return c.elseExpr.Eval(ectx, this)
@@ -864,7 +829,7 @@ func (c *evalCast) Eval(ectx Context, this *zed.Value) *zed.Value {
 	if val.IsNull() || val.Type == c.typ {
 		// If value is null or the type won't change, just return a
 		// copy of the value.
-		return ectx.NewValue(c.typ, val.Bytes)
+		return ectx.NewValue(c.typ, val.Bytes())
 	}
 	return c.caster.Eval(ectx, val)
 }
