@@ -36,7 +36,7 @@ func filter(ectx expr.Context, this *zed.Value, e expr.Evaluator) bool {
 		return true
 	}
 	val := e.Eval(ectx, this)
-	if val.Type == zed.TypeBool && zed.DecodeBool(val.Bytes) {
+	if val.Type == zed.TypeBool && val.Bool() {
 		return true
 	}
 	return false
@@ -67,7 +67,7 @@ func runCasesHelper(t *testing.T, record string, cases []testcase, expectBufferF
 			assert.NoError(t, err, "filter: %q", c.filter)
 			if f != nil {
 				assert.Equal(t, c.expected, filter(expr.NewContext(), rec, f),
-					"filter: %q\nrecord: %s", c.filter, zson.MustFormatValue(rec))
+					"filter: %q\nrecord: %s", c.filter, zson.FormatValue(rec))
 			}
 			bf, err := filterMaker.AsBufferFilter()
 			assert.NoError(t, err, "filter: %q", c.filter)
@@ -77,9 +77,9 @@ func runCasesHelper(t *testing.T, record string, cases []testcase, expectBufferF
 				// hand BufferFilter.Eval a ZNG values frame
 				// containing rec, assembled here.
 				buf := binary.AppendUvarint(nil, uint64(rec.Type.ID()))
-				buf = zcode.Append(buf, rec.Bytes)
+				buf = zcode.Append(buf, rec.Bytes())
 				assert.Equal(t, expected, bf.Eval(zctx, buf),
-					"filter: %q\nvalues:%s\nbuffer:\n%s", c.filter, zson.MustFormatValue(rec), hex.Dump(buf))
+					"filter: %q\nvalues:%s\nbuffer:\n%s", c.filter, zson.FormatValue(rec), hex.Dump(buf))
 			}
 		})
 	}
@@ -362,7 +362,7 @@ func TestFilters(t *testing.T) {
 		{"a != 50.1.168.192", true},
 		{"a in 192.168.0.0/16", false},
 		{"a == 10.0.0.0/16", false},
-		{"a != 192.168.0.0/16", false},
+		{"a != 192.168.0.0/16", true},
 	})
 
 	// Test comparisons with a named type
