@@ -9,7 +9,7 @@ func (c *Coalesce) Call(ctx zed.Allocator, args []zed.Value) *zed.Value {
 	for i := range args {
 		val := &args[i]
 		if !val.IsNull() && !val.IsMissing() && !val.IsQuiet() {
-			return ctx.CopyValue(val)
+			return ctx.CopyValue(*val)
 		}
 	}
 	return zed.Null
