@@ -61,6 +61,7 @@ type Aggregator struct {
 	spiller        *spill.MergeSort
 	partialsIn     bool
 	partialsOut    bool
+	ectx           expr.ResetContext
 }
 
 type Row struct {
@@ -312,11 +313,12 @@ func (a *Aggregator) Consume(batch zbuf.Batch, this *zed.Value) error {
 	// structure at output time, which is the new approach that will be
 	// taken by the fix to #1701.
 
+	a.ectx.SetVars(batch.Vars())
 	types := a.typeCache[:0]
 	keyBytes := a.keyCache[:0]
 	var prim *zed.Value
 	for i, keyExpr := range a.keyExprs {
-		key := keyExpr.Eval(batch, this)
+		key := keyExpr.Eval(a.ectx.Reset(), this)
 		if key.IsQuiet() {
 			return nil
 		}
@@ -326,7 +328,7 @@ func (a *Aggregator) Consume(batch zbuf.Batch, this *zed.Value) error {
 		types = append(types, key.Type)
 		// Append each value to the key as a flat value, independent
 		// of whether this is a primitive or container.
-		keyBytes = zcode.Append(keyBytes, key.Bytes)
+		keyBytes = zcode.Append(keyBytes, key.Bytes())
 	}
 	// We conveniently put the key type code at the end of the key string,
 	// so when we recontruct the key values below, we don't have skip over it.
@@ -350,9 +352,9 @@ func (a *Aggregator) Consume(batch zbuf.Batch, this *zed.Value) error {
 	}
 
 	if a.partialsIn {
-		row.reducers.consumeAsPartial(this, a.aggRefs, batch)
+		row.reducers.consumeAsPartial(this, a.aggRefs, a.ectx.Reset())
 	} else {
-		row.reducers.apply(a.zctx, batch, a.aggs, this)
+		row.reducers.apply(a.zctx, a.ectx.Reset(), a.aggs, this)
 	}
 	return nil
 }
@@ -373,8 +375,9 @@ func (a *Aggregator) spillTable(eof bool, ref zbuf.Batch) error {
 	if err := a.spiller.Spill(a.ctx, recs); err != nil {
 		return err
 	}
+	a.ectx.SetVars(ref.Vars())
 	if !eof && a.inputDir != 0 {
-		val := a.keyExprs[0].Eval(batch, &recs[len(recs)-1])
+		val := a.keyExprs[0].Eval(a.ectx.Reset(), &recs[len(recs)-1])
 		if !val.IsError() {
 			// pass volatile zed.Value since updateMaxSpillKey will make
 			// a copy if needed.
@@ -421,6 +424,9 @@ func (a *Aggregator) readSpills(eof bool, batch zbuf.Batch) (zbuf.Batch, error)
 	if !eof && a.inputDir == 0 {
 		return nil, nil
 	}
+	if batch != nil {
+		a.ectx.SetVars(batch.Vars())
+	}
 	for len(recs) < op.BatchLen {
 		if !eof && a.inputDir != 0 {
 			rec, err := a.spiller.Peek()
@@ -430,12 +436,12 @@ func (a *Aggregator) readSpills(eof bool, batch zbuf.Batch) (zbuf.Batch, error)
 			if rec == nil {
 				break
 			}
-			keyVal := a.keyExprs[0].Eval(batch, rec)
+			keyVal := a.keyExprs[0].Eval(a.ectx.Reset(), rec)
 			if !keyVal.IsError() && a.valueCompare(keyVal, a.maxSpillKey) >= 0 {
 				break
 			}
 		}
-		rec, err := a.nextResultFromSpills(batch)
+		rec, err := a.nextResultFromSpills(a.ectx.Reset())
 		if err != nil {
 			return nil, err
 		}
@@ -487,7 +493,7 @@ func (a *Aggregator) nextResultFromSpills(ectx expr.Context) (*zed.Value, error)
 	for _, e := range a.keyRefs {
 		keyVal := e.Eval(ectx, firstRec)
 		types = append(types, keyVal.Type)
-		a.builder.Append(keyVal.Bytes)
+		a.builder.Append(keyVal.Bytes())
 	}
 	for _, f := range row {
 		var v *zed.Value
@@ -497,7 +503,7 @@ func (a *Aggregator) nextResultFromSpills(ectx expr.Context) (*zed.Value, error)
 			v = f.Result(a.zctx)
 		}
 		types = append(types, v.Type)
-		a.builder.Append(v.Bytes)
+		a.builder.Append(v.Bytes())
 	}
 	typ := a.lookupRecordType(types)
 	bytes, err := a.builder.Encode()
@@ -546,7 +552,7 @@ func (a *Aggregator) readTable(flush, partialsOut bool, batch zbuf.Batch) (zbuf.
 				v = f.Result(a.zctx)
 			}
 			types = append(types, v.Type)
-			a.builder.Append(v.Bytes)
+			a.builder.Append(v.Bytes())
 		}
 		typ := a.lookupRecordType(types)
 		zv, err := a.builder.Encode()
