@@ -7,20 +7,17 @@ import (
 
 type pruner struct {
 	pred expr.Evaluator
-	ectx expr.Context
+	ectx expr.ResetContext
 }
 
 func newPruner(e expr.Evaluator) *pruner {
-	return &pruner{
-		pred: e,
-		ectx: expr.NewContext(),
-	}
+	return &pruner{pred: e}
 }
 
 func (p *pruner) prune(val *zed.Value) bool {
 	if p == nil {
 		return false
 	}
-	result := p.pred.Eval(p.ectx, val)
-	return result.Type == zed.TypeBool && zed.DecodeBool(result.Bytes)
+	result := p.pred.Eval(p.ectx.Reset(), val)
+	return result.Type == zed.TypeBool && result.Bool()
 }
